/****************************************************************)
(*								*)
(*								*)
(*	N       N        AAA       N       N      OOOOO		*)
(*	NN      N       A   A      NN      N     O     O	*)
(*	N N     N      A     A     N N     N    O       O	*)
(*	N  N    N     A       A    N  N    N    O       O	*)
(*	N   N   N     A       A    N   N   N    O       O	*)
(*	N    N  N     AAAAAAAAA    N    N  N    O       O	*)
(*	N     N N     A       A    N     N N    O       O	*)
(*	N      NN     A       A    N      NN     O     O	*)
(*	N       N     A       A    N       N      OOOOO		*)
(*								*)
(*								*)
(****************************************************************)
(*	Compiler fuer ein *sehr* eingeschraenktes Pascal.	*)
(*	Erzeugt Maschinencode fuer 8031 & co Prozessoren.	*)
(****************************************************************/

/*===================================================================*\
|*	$Id: assert.c,v 1.3 2011/08/09 17:25:14 wolf Exp $
|*===================================================================*|
|*
|*	$Log: assert.c,v $
|*	Revision 1.3  2011/08/09 17:25:14  wolf
|*	+ header/footer
|*
|*	Revision 1.2  2011/08/09 17:09:48  wolf
|*	+ Header/footer
|*
\*===================================================================*/

#include "nano.h"
#include <stdio.h>
#include <stdlib.h>

#define MaxAssert 256

static int Buffer[MaxAssert];

static unsigned BaseAddress;
static unsigned Fill;
static unsigned Used;


void
AssertRef (uint16 data) {
	if (Used) {
		fprintf (stderr, "AssertRef: füllt waerend noch nicht ganz verbraucht.\n");
		exit (1);
	}
	if (Fill>=MaxAssert) {
		fprintf (stderr, "AssertRef: Bufferüberlauf.\n");
		exit (1);
	}
	extern uint16 Cadr;
	if (data>=0x100) {
		if (Fill) {
			fprintf (stderr, "AssertRef: Assertadresse für nicht leeren Puffer.\n");
			exit (1);
		}
		Cadr = data;
		return;
	}
	if (!Fill) BaseAddress = Cadr;
	Buffer[Fill++] = data;
}

void
AssertByte (unsigned addr, uint8 byte) {
	if (!Fill) return;

	unsigned idx = addr - BaseAddress;

	if (idx >= Fill) {
		fprintf (stderr, "AssertByte: Adresse %x ausserhalb des Referenzbereiches %x..%x.\n",
			addr, BaseAddress, BaseAddress+Fill-1);
		exit (1);
	}

	if (Buffer[idx]<0) {
		fprintf (stderr, "AssertByte: Adresse %x mehrfach beschrieben.\n", addr);
		exit (1);
	}

	if ((int)(unsigned)byte != Buffer[idx]) {
		fprintf (stderr, "AssertByte: Adresse %x erwartet %02x gefunden %02x.\n", addr, Buffer[idx], byte);
		exit (1);
	}

	Buffer[idx] = -1;
	if (++Used < Fill) return;

	fprintf (stdout, "*** Assertion holds :-) ***\n");

	Used=0;
	Fill=0;
}


/*===================================================================*\
|*	$Id: assert.c,v 1.3 2011/08/09 17:25:14 wolf Exp $
\*===================================================================*/
